@@ -10,7 +10,7 @@ fn parse_time(t: &str) -> Date<Local> {
 fn main() -> Result<(), Box<dyn std::error::Error>> {
     let data = get_data();
     let root = BitMapBackend::new("plotters-doc-data/stock.png", (1024, 768)).into_drawing_area();
-    root.fill(&White)?;
+    root.fill(&WHITE)?;
 
     let (to_date, from_date) = (
         parse_time(&data[0].0) + Duration::days(1),
@@ -23,11 +23,11 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
         .caption("MSFT Stock Price", ("Arial", 50.0).into_font())
         .build_ranged(from_date..to_date, 110f32..135f32)?;
 
-    chart.configure_mesh().line_style_2(&White).draw()?;
+    chart.configure_mesh().line_style_2(&WHITE).draw()?;
 
     chart.draw_series(
         data.iter()
-            .map(|x| CandleStick::new(parse_time(x.0), x.1, x.2, x.3, x.4, &Green, &Red, 15)),
+            .map(|x| CandleStick::new(parse_time(x.0), x.1, x.2, x.3, x.4, &GREEN, &RED, 15)),
     )?;
 
     Ok(())
